package com.caimei.modules.club.controller;

import com.caimei.modules.club.entity.*;
import com.caimei.modules.club.entity.Page;
import com.caimei.modules.club.service.CmUserOrganizeService;
import com.caimei.modules.club.service.CmUserService;
import com.caimei.modules.club.service.NewCmClubService;
import com.caimei.utils.*;
import com.github.pagehelper.PageHelper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;


import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

@Api(tags = "会所管理")
@RestController
@RequiredArgsConstructor
@RequestMapping(value = "/user/cmUserOrganize")
public class CmUserOrganizeController {

    private final CmUserOrganizeService cmUserOrganizeService;
    private final CmUserService cmUserService;
    private final NewCmClubService newCmClubService;

    @ApiOperation("审核机构")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "userOrganizeId", value = "组织Id"),
            @ApiImplicitParam(required = true, name = "auditStatus", value = "审核状态 92审核不通过 90审核通过"),
            @ApiImplicitParam(required = true, name = "clubId", value = "机构Id"),
            @ApiImplicitParam(required = true, name = "userId", value = "用户Id"),
            @ApiImplicitParam(required = true, name = "auditNote", value = "审核备注")
    })
    @GetMapping(value = "/audit")
    public ResponseJson checkClub(CmUser cmUser) {
        if (StringUtils.isBlank(cmUser.getAuditStatus())) {
            return ResponseJson.error("审核状态不能为空！");
        }
        if (null == cmUser.getClubId()) {
            return ResponseJson.error("机构Id不能为空！");
        }
        return cmUserService.auditClubStatus(cmUser);
    }

    @ApiOperation("机构详情")
    @ApiImplicitParam(required = true, name = "clubId", value = "机构Id")
    @GetMapping(value = "/clubDetail/{clubId}")
    public JsonModel userDetail(@PathVariable("clubId") Integer clubId) {
        return JsonModel.newInstance().success(newCmClubService.findClubById(clubId));
    }


    @ApiOperation("机构列表")
    @GetMapping(value = "/clubList")
    public ResponseJson<Page<CmUser>> toCmOrganizeClubList(CmUser cmUser, Page page) {
        if (null == cmUser.getUserOrganizeId()) {
            return ResponseJson.error("组织Id不能为空！");
        }
        PageHelper.startPage(page.getIndex(), page.getPageSize());
        List<CmUser> cmUserList = cmUserService.findListBuyUserInfo(cmUser);
        Page<CmUser> cmUserPage = new Page<>(cmUserList);
        return ResponseJson.success(cmUserPage);
    }

    @ApiOperation("上线会所(添加，编辑)")
    @GetMapping(value = "/toAddClubUser")
    public JsonModel toAddClubUser(NewCmClub newCmClub) {
        if (null != newCmClub.getClubId()) {
            newCmClub = newCmClubService.findClubById(newCmClub.getClubId());
        }
        Integer provinceId = newCmClub.getProvinceId();
        Integer townId = newCmClub.getTownId();
        Integer cityId = newCmClub.getCityId();
        //控制编辑异常时候的会显地址下拉框
        if (null != provinceId && provinceId > 0) {
            Province province = cmUserOrganizeService.loadProvinceById(provinceId);
            newCmClub.setProvince(province.getName());
        }
        if (null != cityId && cityId > 0) {
            City city = cmUserOrganizeService.loadCityById(cityId);
            newCmClub.setCity(city.getName());
        }
        if (null != townId && townId > 0) {
            Town town = cmUserOrganizeService.loadTownById(townId);
            newCmClub.setTown(town.getName());
        }
        return JsonModel.newInstance().success(newCmClub);
    }

    @ApiOperation("保存上线会所信息(添加，编辑)")
    @GetMapping(value = "saveAddClubUser")
    public JsonModel saveAddClubUser(CmUserOrganize cmUserOrganize, NewCmClub newCmClub, CmUser cmUser) throws Exception {
        JsonModel jsonModel = JsonModel.newInstance();
        String contractMobile = newCmClub.getContractMobile();
        Integer userId = newCmClub.getUserId();
        Integer clubId = newCmClub.getClubId();
        cmUser.setUserOrganizeId(cmUserOrganize.getId());
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String currentDate = sdf.format(new Date());
        //判断该会所手机号是否已存在
        cmUser.setBindMobile(contractMobile);
        List<CmUser> listBuyBindMobile = cmUserService.findListBuyBindMobile(cmUser);
        if (null != listBuyBindMobile && listBuyBindMobile.size() > 0) {
            return jsonModel.error("该手机号码已被使用");
        }
        //添加用户信息（新增或修改通用）
        cmUser.setName(newCmClub.getName());
        cmUser.setUserName(newCmClub.getName());
        cmUser.setRealName(newCmClub.getName());
        cmUser.setClubStatus(String.valueOf(newCmClub.getStatus()));
        //添加用户新增信息
        if (null == userId) {
            //新增
            cmUser = initUser(cmUser, newCmClub);
            //插入新的用户数据
            cmUserService.insert(cmUser);
        }
        //添加会所表信息
        newCmClub.setSname(newCmClub.getName());
        if (null == clubId) {
            newCmClub.setUserId(cmUser.getUserId());
            newCmClub.setDefaultServiceProviderId(AppKeys.DEFAULT_SPID);
            newCmClub.setSpId(AppKeys.DEFAULT_SPID);
            newCmClub.setMainServiceProviderId(AppKeys.DEFAULT_SPID);
            newCmClub.setScanFlag("2");
            newCmClub.setFlag("0");
            newCmClub.setAddTime(currentDate);
            newCmClub.setDefaultServiceProviderUpdTime(DateTimeUtil.getCurrentDateTime());
            newCmClubService.insert(newCmClub);

            //更新用户绑定关系
            cmUser.setClubId(newCmClub.getClubId());
            cmUser.setCompanyUserId(cmUser.getUserId());

            //更新会所绑定关系
            newCmClub.setUserId(cmUser.getUserId());
        }
        //更新用户表数据(编辑数据，或新增数据更新)
        cmUserService.update(cmUser);
        //更新会所表数据(编辑数据，或新增数据更新)
        newCmClubService.update(newCmClub);
        return jsonModel.success("保存成功");
    }


    /**
     * 初始化用户数据
     */
    public CmUser initUser(CmUser cmUser, NewCmClub newCmClub) {
        String curTime = DateUtils.getDateTime();
        //设置为会所
        cmUser.setRegisterUserTypeId("3");
        cmUser.setUserIdentity(2);
        //设置默认协销状态
        cmUser.setServiceProviderStatus("90");
        cmUser.setServiceProviderId("1342");
        cmUser.setAuditTime(curTime);
        //审核通过
        cmUser.setAuditStatus("1");
        cmUser.setAuditNote("后台添加用户系统默认审核通过");
        cmUser.setRegisterTime(curTime);
        cmUser.setLoginTime(curTime);
        cmUser.setAccount(newCmClub.getContractMobile() + RandomCodeGenerator.generateAccount(2));
        cmUser.setPassword(MD5Util.md5("caimei123"));
        cmUser.setBindMobile(newCmClub.getContractMobile());
        cmUser.setUserName(newCmClub.getName());
        cmUser.setUserMoney(0d);
        cmUser.setAbleUserMoney(0d);
        cmUser.setUserBeans("0");
        cmUser.setValidFlag(1);
        cmUser.setAgreeFlag(1);
        cmUser.setUserPermission(2);
        return cmUser;
    }

    @ApiOperation("上下线会所")
    @GetMapping(value = "switchLine")
    public JsonModel switchLine(CmUser cmUser) {
        if ("91".equals(cmUser.getClubStatus())) {
            cmUser.setClubStatus("90");
        } else {
            cmUser.setClubStatus("91");
        }
        //更新用户表数据
        cmUserService.update(cmUser);
        NewCmClub newCmClub = new NewCmClub();
        newCmClub.setClubId(cmUser.getClubId());
        newCmClub.setUserId(cmUser.getUserId());
        newCmClub.setStatus(Integer.valueOf(cmUser.getClubStatus()));
        //更新会所表数据
        newCmClubService.update(newCmClub);
        return JsonModel.newInstance().success();
    }

    @ApiOperation("省")
    @GetMapping("/province")
    public JsonModel getProvince() {
        List<Province> provinceList = cmUserOrganizeService.getProvince();
        return JsonModel.newInstance().success(provinceList);
    }

    @ApiOperation("市")
    @GetMapping("/city")
    public JsonModel getCity(Long provinceid) {
        List<City> cityList = cmUserOrganizeService.getCity(provinceid);
        return JsonModel.newInstance().success(cityList);
    }

    @ApiOperation("区")
    @GetMapping("/town")
    public JsonModel getTown(Integer cityid) {
        List<Town> townList = cmUserOrganizeService.getTown(cityid);
        return JsonModel.newInstance().success(townList);
    }

}