package com.caimei.modules.order.controller;


import com.caimei.modules.order.entity.CmOrderRemark;
import com.caimei.modules.order.entity.NewOrder;
import com.caimei.modules.order.entity.NewShopOrder;
import com.caimei.modules.order.service.CmOrderRemarkService;
import com.caimei.modules.order.service.NewOrderService;
import com.caimei.modules.order.util.OrderUtils;
import com.caimei.utils.ResponseJson;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


@Api(tags = "订单备注api")
@RestController
@RequiredArgsConstructor
@RequestMapping(value = "/order/cmOrderRemark")
public class CmOrderRemarkController {


    private final CmOrderRemarkService cmOrderRemarkService;
    private final NewOrderService newOrderService;
    private final OrderUtils orderUtil;

    @ApiOperation("订单备注详情")
    @GetMapping(value = "/remarksView/{orderId}/{shopOrderId}")
    public ResponseJson<List<CmOrderRemark>> remarksView(@PathVariable("orderId") Integer orderId,@PathVariable("shopOrderId") Integer shopOrderId) {
        List<CmOrderRemark> remarksList = cmOrderRemarkService.findList(shopOrderId);
        NewOrder newOrder = newOrderService.get(orderId.toString());
        orderUtil.setOrderValue(newOrder);
        List<NewShopOrder>  newShopOrders= newOrder.getNewShopOrders().stream().filter(newShopOrder -> newShopOrder.getShopOrderId().equals(shopOrderId)).collect(Collectors.toList());
        newOrder.setNewShopOrders(newShopOrders);
        if(remarksList!=null&&remarksList.size()>0){

            remarksList.get(0).setNewOrder(newOrder);
        }
        return ResponseJson.success(remarksList);
    }

    @ApiOperation("添加订单备注")
    @PostMapping(value = "/add/remark")
    public ResponseJson addRemark(@RequestBody CmOrderRemark remark) {
        cmOrderRemarkService.insertRemark(remark);
        return ResponseJson.success();
    }

    @ApiOperation("上传备注文件")
    @PostMapping("/upload/remark")
    public ResponseJson<Map<String, Object>> uploadFile(MultipartFile file, Integer orderId) {
        return cmOrderRemarkService.upload(file, orderId);
    }

    @ApiOperation("下载备注文件")
    @GetMapping("/download/remark")
    public void downFile(Integer fileId, HttpServletRequest request, HttpServletResponse response) throws IOException {
        cmOrderRemarkService.downFile(fileId, request, response);
    }
}
