package com.caimei.modules.order.controller;

import com.caimei.modules.order.entity.CompanyVo;
import com.caimei.modules.order.entity.DeliverDto;
import com.caimei.modules.order.entity.LogisticsInfoVo;
import com.caimei.modules.order.entity.NewOrder;
import com.caimei.modules.order.service.OrderDeliverService;
import com.caimei.utils.ResponseJson;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author zzj
 */
@Api(tags = "订单发货api")
@ApiSupport(author = "zzj")
@RestController
@RequiredArgsConstructor
@RequestMapping(value = "/order/deliver")
public class OrderDeliverController {

    @Resource
    private OrderDeliverService deliverService;

    @ApiOperation("去发货")
    @GetMapping("/to/deliver/{orderId}")
    public ResponseJson<NewOrder> toDeliver(@PathVariable("orderId") Integer orderId) {
        return ResponseJson.success(deliverService.toDeliver(orderId));
    }

    @ApiOperation("确认发货")
    @PostMapping("/confirm/deliver")
    public ResponseJson confirmDeliver(@RequestBody DeliverDto dto) {
        return deliverService.confirmDeliver(dto);
    }

    @ApiOperation("发货记录")
    @GetMapping("/deliver/details/{orderId}/{shopOrderId}")
    public ResponseJson<List<LogisticsInfoVo>> logisticsDetails(@PathVariable("orderId") Integer orderId,@PathVariable("shopOrderId") Integer shopOrderId) {
        return deliverService.findDetails(orderId,shopOrderId);
    }

    @ApiOperation("物流公司")
    @ApiImplicitParam(required = false, name = "value", value = "快递公司")
    @GetMapping("/ship/company")
    public ResponseJson<List<CompanyVo>> getLogisticsCompany(String value) {
        return deliverService.getLogisticsCompany(value);
    }
}
