package com.caimei.modules.order.controller;

import com.caimei.modules.order.entity.*;
import com.caimei.modules.order.service.PayShopService;
import com.caimei.utils.ResponseJson;
import com.github.pagehelper.PageInfo;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.util.List;
import java.util.Map;

/**
 * @author zzj
 */
@Api(tags = "订单付款API")
@ApiSupport(author = "zzj")
@RestController
@RequiredArgsConstructor
@RequestMapping(value = "/order/organize")
public class OrderPayShopController {

    private final PayShopService payShopService;

    @ApiOperation("付款列表")
    @GetMapping("/pay/list")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "orderId", value = "订单Id"),
            @ApiImplicitParam(required = false, name = "orderNo", value = "订单编号"),
            @ApiImplicitParam(required = false, name = "shopOrderId", value = "子订单Id"),
            @ApiImplicitParam(required = false, name = "shopOrderNo", value = "子订单编号"),
            @ApiImplicitParam(required = false, name = "status", value = "审核状态  0待审核,  1审核通过 2 审核不通过"),
            @ApiImplicitParam(required = false, name = "applyStartTime", value = "申请起始时间"),
            @ApiImplicitParam(required = false, name = "applyEndTime", value = "申请终止时间"),
            @ApiImplicitParam(required = false, name = "payStartTime", value = "付款起始时间"),
            @ApiImplicitParam(required = false, name = "payEndTime", value = "付款终止时间"),
            @ApiImplicitParam(required = false, name = "shopName", value = "供应商名称"),
            @ApiImplicitParam(required = false, name = "name", value = "付款单名称"),
            @ApiImplicitParam(required = true, name = "organizeId", value = "组织Id"),
            @ApiImplicitParam(required = true, name = "pageNum", value = "当前页码"),
            @ApiImplicitParam(required = true, name = "pageSize", value = "每页大小")
    })
    public ResponseJson<PageInfo<OrderPayShop>> payList(OrderPayShop orderPayShop,
                                                        @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                        @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        return ResponseJson.success(payShopService.findPayList(orderPayShop, pageNum, pageSize));
    }

    @ApiOperation("申请付款列表")
    @GetMapping("/payOrder/list")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "orderId", value = "订单Id"),
            @ApiImplicitParam(required = false, name = "orderNo", value = "订单编号"),
            @ApiImplicitParam(required = false, name = "shopOrderId", value = "子订单Id"),
            @ApiImplicitParam(required = false, name = "shopOrderNo", value = "子订单编号"),
            @ApiImplicitParam(required = false, name = "orderStartTime", value = "下单起始时间"),
            @ApiImplicitParam(required = false, name = "orderEndTime", value = "下单终止时间"),
            @ApiImplicitParam(required = false, name = "shopName", value = "供应商名称"),
            @ApiImplicitParam(required = false, name = "clubName", value = "机构名称"),
            @ApiImplicitParam(required = true, name = "listType", value = "列表类型 1申请付款 2子订单分账 3子订单结算"),
            @ApiImplicitParam(required = true, name = "organizeId", value = "组织Id"),
            @ApiImplicitParam(required = true, name = "pageNum", value = "当前页码"),
            @ApiImplicitParam(required = true, name = "pageSize", value = "每页大小")
    })
    public ResponseJson<PageInfo<NewShopOrder>> payOrderList(OrderPayShop orderPayShop,
                                                             @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                             @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        return ResponseJson.success(payShopService.findPayOrderList(orderPayShop, pageNum, pageSize));
    }

    @ApiOperation("付款单详情")
    @ApiImplicitParam(required = true, name = "id", value = "付款单Id")
    @GetMapping("/pay/form/{id}")
    public ResponseJson<OrderPayShopDetail> payDetail(@PathVariable("id") Integer id) {
        return ResponseJson.success(payShopService.getPayShopDetail(id));
    }

    @ApiOperation("批量申请付款单")
    @ApiImplicitParam(required = true, name = "shopOrderIds", value = "子订单IdList")
    @PostMapping("/apply/pays")
    public ResponseJson<OrderPayShopDetail> applyPayList(@RequestBody List<Integer> shopOrderIds) {
        return payShopService.applyPayList(shopOrderIds);
    }

    @ApiOperation("审核付款单")
    @PostMapping("/pay/check")
    public ResponseJson checkPass(@RequestBody PayCheckDto checkDto) throws Exception {
        return payShopService.checkPass(checkDto.getId(), checkDto.getPayType(), checkDto.getPassCode(), checkDto.getReason());
    }

    @ApiOperation("付款申请/重申付款")
    @PostMapping("/pay/apply")
    public ResponseJson<Integer> applyPay(@RequestBody OrderPayShopDetail payShop) throws Exception {
        return payShopService.applyPay(payShop);
    }

    @ApiOperation("取消付款单（待审核可以取消）")
    @ApiImplicitParam(required = true, name = "id", value = "付款单Id")
    @GetMapping("/pay/cancel/{id}")
    public ResponseJson cancelPay(@PathVariable("id") Integer id) {
        return payShopService.cancelPay(id);
    }

    @ApiOperation("撤销付款单(审核通过可以撤销)")
    @GetMapping("/pay/back/{id}")
    public ResponseJson backPay(@PathVariable("id") Integer id) {
        return payShopService.backPay(id);
    }

    @ApiOperation("申请分账")
    @ApiImplicitParam(name = "shopOrderIds", value = "子订单IdList", required = true, allowMultiple = true)
    @PostMapping("/pay/split")
    public ResponseJson splitShopOrders(@RequestBody List<Integer> shopOrderIds) {
        return payShopService.splitShopOrders(shopOrderIds);
    }

    @ApiOperation("申请结算")
    @PostMapping("/pay/settle")
    public ResponseJson settleShopOrders(@RequestBody OrderPayShopDetail payShop) {
        return payShopService.settleShopOrders(payShop);
    }

    @ApiOperation("订单财务信息列表")
    @GetMapping("/pay/infos")
    public ResponseJson<PageInfo<NewOrder>> orderPayInfoList(OrderPayShop orderPayShop,
                                                             @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                             @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        return payShopService.orderPayInfoList(orderPayShop, pageNum, pageSize);
    }

    @ApiOperation("订单分账记录")
    @ApiImplicitParam(name = "orderId", value = "订单id", required = true)
    @GetMapping("/split/record/{orderId}")
    public ResponseJson<Map<String, Object>> findSplitRecord(@PathVariable Integer orderId) {
        return payShopService.findSplitRecord(orderId);
    }

    @ApiOperation("订单结算记录")
    @ApiImplicitParam(name = "orderId", value = "订单id", required = true)
    @GetMapping("/settle/record/{orderId}")
    public ResponseJson<Map<String, Object>> findSettleRecord(@PathVariable Integer orderId) {
        return payShopService.findSettleRecord(orderId);
    }

    @ApiOperation("订单收付款记录")
    @ApiImplicitParam(name = "orderId", value = "订单id", required = true)
    @GetMapping("/receipt/record/{orderId}/{shopOrderId}")
    public ResponseJson<Map<String, Object>> findReceiptRecord(@PathVariable Integer orderId,@PathVariable Integer shopOrderId) {
        return payShopService.findReceiptRecord(orderId,shopOrderId);
    }
}
