package com.caimei.modules.order.controller;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.caimei.modules.club.entity.Page;
import com.caimei.modules.order.dao.NewOrderDao;
import com.caimei.modules.order.entity.*;
import com.caimei.modules.order.service.*;
import com.caimei.modules.order.util.OrderUtils;
import com.caimei.utils.JsonModel;
import com.caimei.utils.ResponseJson;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.junit.Assert;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Api(tags = "订单api")
@RestController
@RequiredArgsConstructor
@RequestMapping(value = "/order/organize")
public class OrganizeOrderController {

    private final NewOrderService newOrderService;

    private final BpClauseService bpClauseService;

    private final CmReturnedPurchaseService cmReturnedPurchaseService;

    private final CmReceiptOrderRelationService cmReceiptOrderRelationService;

    private final CmDiscernReceiptService cmDiscernReceiptService;

    private final OrderUtils orderUtil;

    private final NewOrderDao orderDao;

    @ModelAttribute
    public NewOrder getOrder(@RequestParam(required = false) String id) {
        NewOrder entity = null;
        if (StringUtils.isNotBlank(id)) {
            entity = newOrderService.get(id);
        }
        if (entity == null) {
            entity = new NewOrder();
        }
        return entity;
    }

    @ApiOperation("更改收款状态")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "orderId", value = "订单Id"),
            @ApiImplicitParam(required = true, name = "amount", value = "本次收款金额")
    })
    @GetMapping("/check/receipt")
    public ResponseJson checkReceipt(Double amount, Integer orderId) {
        if (null == amount || null == orderId) {
            return ResponseJson.error("参数不能为空！");
        }
        return newOrderService.checkReceipt(amount, orderId);
    }

    /**
     * 查询参数 ：订单Id,订单编号,下单时间/终止时间,买家,收货人,订单状态,收款状态,发货状态,退款状态,待审核退款
     */
    @ApiOperation("订单列表")
    @GetMapping("orderList")
    public JsonModel orderList(NewOrder order, Page page) {
        Assert.assertNotNull("组织id不能为空！", order.getOrganizeId());
        if (null != order.getBuyer()) {
            order.setBuyer(order.getBuyer().trim());
        }
        if (null != order.getReceiver()) {
            order.setReceiver(order.getReceiver().trim());
        }
        //查询内部处理了已收,未付统计（包含运费）
        Page<NewOrder> orderPage = newOrderService.findList(order, page);
        return JsonModel.newInstance().success(orderPage);
    }

    @ApiOperation("订单详情")
    @GetMapping("detail")
    public JsonModel detail(NewOrder order) {
        Map<String, Object> map = new HashMap();
        BpClause bpClause = new BpClause();
        bpClause.setEnabledStatus("1");
        List<BpClause> bpClauses = bpClauseService.findList(bpClause);
        map.put("bpClauses", bpClauses);
        orderUtil.setOrderValue(order);
//        orderUtil.setOrderPostInfo(order);

        /**
         *  当主订单为【待收款】或者为线下支付的【部分收款】状态时，才显示更改订单状态按钮
         *  （排除交易全退订单）
         */
        Integer checkFlag = 0;
        if ("11".equals(order.getStatus()) || "12".equals(order.getStatus()) || "13".equals(order.getStatus())) {
            checkFlag = 1;
        } else if ("21".equals(order.getStatus()) || "22".equals(order.getStatus()) || "23".equals(order.getStatus())) {
            Integer count = orderDao.findOnlinePay(order.getOrderId());
            checkFlag = count == 0 ? 1 : 0;
        } else {
            checkFlag = 0;
        }
        order.setCheckFlag(checkFlag);
        map.put("order", order);

        List<Voucher> vouchers = newOrderService.findVoucher(order.getShopOrderId());
        map.put("voucher", vouchers);
        CmReturnedPurchase cmReturnedPurchase = new CmReturnedPurchase();
        cmReturnedPurchase.setStatus("2");
        cmReturnedPurchase.setOrderId(order.getOrderId());
        List<CmReturnedPurchase> dataList = cmReturnedPurchaseService.getReturnedPurchaseList(cmReturnedPurchase);

        //退款(退货)总金额
        Double returnedPurchaseFee = 0D;
        //退款总额(给买家)
        Double refundFee = 0D;
        for (CmReturnedPurchase bean : dataList) {
            if (!"3".equals(bean.getStatus())) {
                returnedPurchaseFee += bean.getReturnedPurchaseFee();
                refundFee += bean.getRefundFee();
            }
        }
        //退货价值
        Double returnValue = 0d;
        if (order.getDiscountFee() >= returnedPurchaseFee) {
            //如果经理折扣大于退款(退货)总金额之和  那么先抵扣经理折扣  退款价值为0
            returnValue = 0d;
        } else {
            //抵用完经理折扣后才是真实价值
            returnValue = returnedPurchaseFee - order.getDiscountFee();
        }
        //订单已收金额
        Double receiptAmount = cmDiscernReceiptService.findOrderReceipt(order.getOrderId());
        map.put("returnValue", returnValue);
        map.put("receiptAmount", receiptAmount == null ? 0 : receiptAmount);
        map.put("returnedPurchaseFee", returnedPurchaseFee);
        return JsonModel.newInstance().success(map);
    }

    @ApiOperation("收退款记录")
    @GetMapping(value = "toRefundRecord")
    public JsonModel toRefundRecord(CmReturnedPurchase cmReturnedPurchase, String from) {
        JsonModel jsonModel = JsonModel.newInstance();
        Map<String, Object> map = new HashMap(6);
        NewOrder newOrder = newOrderService.get(cmReturnedPurchase.getOrderId().toString());
        orderUtil.setOrderPostInfo(newOrder);
        List<CmReceiptOrderRelation> listRelation = cmReceiptOrderRelationService.findByOrderId(newOrder.getOrderId());
        List<CmReturnedPurchase> dataList = cmReturnedPurchaseService.getReturnedPurchaseList(cmReturnedPurchase);
        //退款总额
        Double refundFee = 0D;
        for (CmReturnedPurchase bean : dataList) {
            if (!"3".equals(bean.getStatus())) {
                refundFee += bean.getRefundFee();
            }
        }
        map.put("cmReturnedPurchaseList", dataList);
        map.put("order", newOrder);
        map.put("receipt", listRelation);
        map.put("refundFee", refundFee);
        map.put("from", from);
        map.put("orderId", cmReturnedPurchase.getOrderId().toString());
        return jsonModel.success(map);
    }

    @ApiOperation("退款退货记录")
    @GetMapping(value = "toRecturnRecordByOrderId")
    public JsonModel toRecturnRecordByOrderId(CmReturnedPurchase cmReturnedPurchase) {
        Map<String, Object> map = new HashMap();
        try {
            NewOrder newOrder = newOrderService.get(cmReturnedPurchase.getOrderId().toString());
//            orderUtil.setOrderPostInfo(newOrder);
            //过滤子订单
            orderUtil.setOrderValue(newOrder);
            List<NewShopOrder>  newShopOrders= newOrder.getNewShopOrders().stream().filter(newShopOrder -> newShopOrder.getShopOrderId().equals(cmReturnedPurchase.getShopOrderId())).collect(Collectors.toList());
            newOrder.setNewShopOrders(newShopOrders);


            List<CmReturnedPurchase> dataList = cmReturnedPurchaseService.getReturnedPurchaseList(cmReturnedPurchase).stream().filter(returnedPurchase -> returnedPurchase.getShopOrderId() .equals( cmReturnedPurchase.getShopOrderId())).collect(Collectors.toList());
            //退款(退货)总金额
            Double returnedPurchaseFee = 0D;
            //退款总额(给买家)
            Double refundFee = 0D;
            for (CmReturnedPurchase bean : dataList) {
                if (!bean.getStatus().equals("3")) {
                    returnedPurchaseFee += bean.getReturnedPurchaseFee();
                    refundFee += bean.getRefundFee();
                }
            }
            BpClause bpClause = new BpClause();
            bpClause.setEnabledStatus("1");
            List<BpClause> bpClauses = bpClauseService.findList(bpClause);
            map.put("bpClauses", bpClauses);
            map.put("cmReturnedPurchaseList", dataList);
            map.put("order", newOrder);
            map.put("returnedPurchaseFee", returnedPurchaseFee);
            map.put("refundFee", refundFee);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return JsonModel.newInstance().success(map);
    }

    /**
     * 保存支付凭证
     * @param voucher
     * @return
     */
    @PostMapping(value = "/saveVoucher")
    public JsonModel saveVoucher(@RequestBody Voucher voucher) {
        if (null == voucher.getOrderId()) {
            return JsonModel.newInstance().error(-1,null,"订单Id不能为空");
        }
        newOrderService.saveVoucher(voucher);
        return JsonModel.newInstance().success();
    }
}
