package com.caimei.modules.order.service.impl;

import com.caimei.modules.order.dao.CmDiscernReceiptDao;
import com.caimei.modules.order.entity.*;
import com.caimei.modules.order.service.CmDiscernReceiptService;
import com.caimei.modules.order.service.NewOrderService;
import com.caimei.utils.MathUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

@Slf4j
@Service
public class CmDiscernReceiptServiceImpl implements CmDiscernReceiptService {

    @Resource
    private CmDiscernReceiptDao cmDiscernReceiptDao;
    @Resource
    private NewOrderService orderService;


    @Override
    public Double findOrderReceipt(Integer orderId) {
        return cmDiscernReceiptDao.findOrderReceipt(orderId);
    }

    @Override
    public PageInfo<ReceiptVo> findReceiptList(ReceiptVo receiptVo, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<ReceiptVo> receiptList = cmDiscernReceiptDao.findReceiptList(receiptVo);
        PageInfo<ReceiptVo> pageInfo = new PageInfo(receiptList);
        return pageInfo;
    }

    @Override
    public ReceiptDetail findReceiptDetail(Integer id,Integer shopOrderId) {
        ReceiptDetail r = cmDiscernReceiptDao.findReceiptDetail(id);
        List<ReceiptOrderInfo> orderInfos = cmDiscernReceiptDao.findReceiptOrder(r.getOrderId());
        orderInfos.forEach(o -> {
            o.setRecords(cmDiscernReceiptDao.findOrderRelation(r.getReceiptId()));
//            if (1 == r.getPayWay()) {
            List<NewShopOrder> receiptShopOrder = cmDiscernReceiptDao.findReceiptShopOrder(o.getOrderId(), shopOrderId);
            receiptShopOrder.forEach(newShopOrder -> {
                BigDecimal add = MathUtil.add(newShopOrder.getReceiptAmount(), cmDiscernReceiptDao.findShoOrderIDReceipt(newShopOrder.getShopOrderId()));
                newShopOrder.setReceiptAmount(add.doubleValue());
                BigDecimal sub = MathUtil.sub(
                        newShopOrder.getRealPay() != null ? newShopOrder.getRealPay() : newShopOrder.getNeedPayAmount()
                        , newShopOrder.getReceiptAmount());
                newShopOrder.setRestAmount(sub.doubleValue());
            });

            o.setShopOrders(receiptShopOrder);
//            }
        });
        r.setOrderInfos(orderInfos);
        if (2 == r.getPayWay()) {
            List<Voucher> vouchers = orderService.findVoucher(shopOrderId);
            r.setVouchers(vouchers);
        }
        return r;
    }

    @Override
    public List<ReceiptPayType> findPayType() {
        return cmDiscernReceiptDao.findReceiptType();
    }

    @Override
    public List<ReceiptPayType> payType() {
        return cmDiscernReceiptDao.findPayType();
    }
}
