package com.caimei.modules.order.service.impl;

import java.util.Date;
import java.util.List;

import com.aliyun.oss.common.utils.StringUtils;
import com.caimei.modules.order.dao.CmUserBalanceRecordMapper;
import com.caimei.modules.order.entity.CmUserBalanceRecord;
import com.caimei.modules.order.service.CmUserBalanceRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 用户余额收支记录Service业务层处理
 *
 * @author Kaick
 * @date 2023-06-18
 */
@Service
public class CmUserBalanceRecordServiceImpl implements CmUserBalanceRecordService {
    @Autowired
    private CmUserBalanceRecordMapper cmUserBalanceRecordMapper;

    /**
     * 查询用户余额收支记录
     *
     * @param id 用户余额收支记录主键
     * @return 用户余额收支记录
     */
    @Override
    public CmUserBalanceRecord getCmUserBalanceRecordById(Integer id) {
        return cmUserBalanceRecordMapper.getCmUserBalanceRecordById(id);
    }

    /**
     * 查询用户余额收支记录
     *
     * @param cmUserBalanceRecord 用户余额收支记录
     * @return 用户余额收支记录
     */
    @Override
    public CmUserBalanceRecord getByCmUserBalanceRecord(CmUserBalanceRecord cmUserBalanceRecord) {
        return cmUserBalanceRecordMapper.getByCmUserBalanceRecord(cmUserBalanceRecord);
    }

    /**
     * 查询用户余额收支记录列表
     *
     * @param cmUserBalanceRecord 用户余额收支记录
     * @return 用户余额收支记录
     */
    @Override
    public List<CmUserBalanceRecord> getCmUserBalanceRecordList(CmUserBalanceRecord cmUserBalanceRecord) {
        return cmUserBalanceRecordMapper.getCmUserBalanceRecordList(cmUserBalanceRecord);
    }

    /**
     * 新增用户余额收支记录
     *
     * @param cmUserBalanceRecord 用户余额收支记录
     * @return 结果
     */
    @Override
    public int addCmUserBalanceRecord(CmUserBalanceRecord cmUserBalanceRecord) {
        return cmUserBalanceRecordMapper.addCmUserBalanceRecord(cmUserBalanceRecord);
    }

    /**
     * 修改用户余额收支记录
     *
     * @param cmUserBalanceRecord 用户余额收支记录
     * @return 结果
     */
    @Override
    public int updateCmUserBalanceRecord(CmUserBalanceRecord cmUserBalanceRecord) {
        return cmUserBalanceRecordMapper.updateCmUserBalanceRecord(cmUserBalanceRecord);
    }

    /**
     * 批量删除用户余额收支记录
     *
     * @param ids 需要删除的用户余额收支记录主键
     * @return 结果
     */
    @Transactional
    @Override
    public int delCmUserBalanceRecordByIds(Integer[] ids) {
        return cmUserBalanceRecordMapper.updateDelCmUserBalanceRecordByIds(ids);
        //return cmUserBalanceRecordMapper.delCmUserBalanceRecordByIds(ids);
    }

    /**
     * 删除用户余额收支记录信息
     *
     * @param id 用户余额收支记录主键
     * @return 结果
     */
    @Transactional
    @Override
    public int delCmUserBalanceRecordById(Integer id) {
        return cmUserBalanceRecordMapper.updateCmUserBalanceRecord(new CmUserBalanceRecord().setId(id).setDelFlag("1"));
        //return cmUserBalanceRecordMapper.delCmUserBalanceRecordById(id);
    }

    /**
     * 批量新增用户余额收支记录
     *
     * @param cmUserBalanceRecordList 用户余额收支记录列表
     * @return 结果
     */
    @Transactional
    @Override
    public void batchAddCmUserBalanceRecord(List<CmUserBalanceRecord> cmUserBalanceRecordList) {
        if (null != cmUserBalanceRecordList) {
            for (CmUserBalanceRecord cmUserBalanceRecord : cmUserBalanceRecordList) {
                if (null != cmUserBalanceRecord) {
                    cmUserBalanceRecordMapper.addCmUserBalanceRecord(cmUserBalanceRecord);
                }
            }
        }
    }

    /**
     * 批量修改用户余额收支记录
     *
     * @param cmUserBalanceRecordList 用户余额收支记录列表
     * @return 结果
     */
    @Transactional
    @Override
    public void batchUpdateCmUserBalanceRecord(List<CmUserBalanceRecord> cmUserBalanceRecordList) {
        if (null != cmUserBalanceRecordList) {
            for (CmUserBalanceRecord cmUserBalanceRecord : cmUserBalanceRecordList) {
                if (null != cmUserBalanceRecord) {
                    cmUserBalanceRecordMapper.updateCmUserBalanceRecord(cmUserBalanceRecord);
                }
            }
        }
    }
}
