package com.caimei.modules.order.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.caimei.modules.club.entity.Page;
import com.caimei.modules.order.dao.*;
import com.caimei.modules.order.entity.*;
import com.caimei.modules.order.service.NewOrderService;
import com.caimei.modules.order.service.OrderUserInfoService;
import com.caimei.modules.order.util.OrderUtils;
import com.caimei.utils.AppKeys;
import com.caimei.utils.AppUtils;
import com.caimei.utils.NewOrderStatus;
import com.caimei.utils.ResponseJson;
import com.github.pagehelper.PageHelper;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

@Service
public class NewOrderServiceImpl implements NewOrderService {

    @Resource
    private NewOrderDao newOrderDao;
    @Resource
    private NewShopOrderDao newShopOrderDao;
    @Resource
    private OrderUserInfoService orderUserInfoService;
    @Resource
    private NewOrderProductDao newOrderProductDao;
    @Resource
    private CmPayShopRecordDao cmPayShopRecordDao;
    @Resource
    private OrderInvoiceDao orderInvoiceDao;
    @Resource
    private OrderUtils orderUtil;
    @Value("${malladmin.domain}")
    private String domain;

    @Override
    public Page<NewOrder> findList(NewOrder newOrder, Page page) {
        PageHelper.startPage(page.getIndex(), page.getPageSize());
        List<NewOrder> orderList = newOrderDao.findList(newOrder);
        orderList.forEach(o->orderUtil.setOrderValue(o));
        return new Page<>(orderList);
    }

    @Override
    public NewOrder get(String id) {
        NewOrder newOrder = newOrderDao.get(id);
        if (null != newOrder) {
            //获取订单客户
            OrderUserInfo orderUserInfo = orderUserInfoService.findByOrderId(newOrder.getOrderId());
            if (null != orderUserInfo) {
                newOrder.setBpOrderUserinfo(orderUserInfo);
            }
            //获取订单商品
            List<NewOrderProduct> list = new ArrayList<>();
            List<NewShopOrder> shopOrders = newShopOrderDao.findListByOrderId(newOrder.getOrderId());
            AtomicReference<Double> freight = new AtomicReference<>(0D);
            AtomicReference<Double> totalAddedValueTax = new AtomicReference<>(0D);
            if (CollectionUtils.isNotEmpty(shopOrders)) {
                for (NewShopOrder shopOrder : shopOrders) {
                    List<NewOrderProduct> pList = newOrderProductDao.findListByShopOrderId(shopOrder.getShopOrderId());
                    list.addAll(pList);
                    shopOrder.setNewOrderProducts(pList);
                    if (pList.stream().anyMatch(p -> p.getProductId() == 999)) {
                        shopOrder.setPostFeeCos("1");
                    }
                    pList.forEach(item -> {
                        if (NewOrderStatus.hasReturn(newOrder.getStatus())) {
                            Integer returningNum = newOrderProductDao.getReturningNum(item.getOrderProductId());
                            Integer returnedNum = newOrderProductDao.getReturnedNum(item.getOrderProductId());
                            item.setReturningNum(returningNum);
                            item.setReturnedNum(returnedNum);
                        }
                        /**确认收货信息*/
                        if (NewOrderStatus.hasReceived(newOrder.getStatus())) {
                            Integer receivedNum = newOrderProductDao.getReceivedNum(item.getOrderProductId());
                            item.setReceivedNum(receivedNum);
                        }
                        if (item.getProductId() == 999) { //邮费商品
                            freight.updateAndGet(v -> (v + item.getPrice()));
                        }
                        totalAddedValueTax.updateAndGet(v -> (v + item.getTotalAddedValueTax()));
                        item.setProductImage(AppUtils.getProductImageURL(item.getProductImage(), AppKeys.ORIGINAL_PICTURE, domain));
                    });
                    Double payed = cmPayShopRecordDao.sumByShopOrderId(shopOrder.getShopOrderId());
                    shopOrder.setPayed(payed);
                }
                newOrder.setNewShopOrders(shopOrders);
            }
            if (CollectionUtils.isNotEmpty(list)) {
                newOrder.setOrderProduct(list);
            }
            if ("1".equals(newOrder.getFreePostFlag())) {
                newOrder.setFreight(freight.get());
            }
            newOrder.setTotalAddedValueTax(totalAddedValueTax.get());
            shopOrders.sort((so1, so2) -> {
                //给子订单排序  邮费排在最后  其余的按shopOrderId  asc
                if ("1".equals(so1.getPostFeeCos())) {
                    return 1;
                }
                return so1.getShopOrderId() - so2.getShopOrderId();
            });
            //获取订单发票
            OrderInvoice orderInvoice = orderInvoiceDao.getWithorderId(newOrder.getOrderId() + "");
            if (null != orderInvoice) {
                newOrder.setOrderInvoice(orderInvoice);
            }
        }
        return newOrder;
    }

    @Override
    @Transactional(readOnly = false, rollbackFor = Exception.class)
    public ResponseJson checkReceipt(Double amount,Integer orderId) {
        return orderUtil.addReceiptRecord(amount, orderId);
    }

    @Override
    public List<Voucher> findVoucher(Integer shopOrderID) {
        List<Voucher> voucher = newOrderDao.findVoucher(shopOrderID);
        voucher.forEach(v -> v.setImages(newOrderDao.findVocherImage(v.getId())));
        return voucher;
    }

    @Override
    public void saveVoucher(Voucher voucher) {
        // 保存线下支付凭证记录
        newOrderDao.insertVoucher(voucher);
        // 保存线下支付凭证图
        for (String str: voucher.getImages()) {
            newOrderDao.insertVoucherImage(voucher.getId(), str);
        }
    }
}
