package com.caimei.modules.shiro.controller;

import com.caimei.modules.club.entity.Page;
import com.caimei.modules.shiro.dao.MallUserDao;
import com.caimei.modules.shiro.entity.CmMallAdminUser;
import com.caimei.modules.shiro.service.MallUserService;
import com.caimei.utils.JsonModel;
import com.caimei.utils.ResponseJson;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Description
 *
 * @author : Charles
 * @date : 2023/5/25
 */
@RestController
@RequestMapping("/other/user")
@Slf4j
public class MallUserController {

    @Autowired private MallUserService mallUserService;
    @Resource
    private MallUserDao mallUserDao;

    /**
     * 后台用户列表
     * @param cmMallAdminUser
     * @return
     */
    @GetMapping(value = "/getMallUserInfo")
    public ResponseJson<Page<CmMallAdminUser>> getMallUserInfo(CmMallAdminUser cmMallAdminUser,
                                                               @RequestParam(value = "pageNum" , defaultValue = "1") Integer pageNum,
                                                               @RequestParam(value = "pageSize", defaultValue = "20") Integer pageSize) {
        return mallUserService.getMallUserInfo(cmMallAdminUser,pageNum, pageSize);
    }

    /**
     * 修改上下线状态
     * @param status
     * @param id
     * @return
     */
    @GetMapping(value = "/updateUser")
    public ResponseJson updateUser(Integer status, Integer id) {
        if (null == status) {
            return ResponseJson.error(-1,"状态不能为空",null);
        }
        if (null == id) {
            return ResponseJson.error(-1,"用户Id不能为空",null);
        }
        mallUserService.updateUser(status, id);
        return ResponseJson.success();
    }

    /**
     * 用户信息回显
     * @param id
     * @return
     */
    @GetMapping(value = "/mallUserById/{id}")
    public ResponseJson getMallUserById(@PathVariable("id") Integer id) {

        CmMallAdminUser mallUserById = mallUserService.getMallUserById(id);
        return ResponseJson.success(mallUserById);
    }

    /**
     * 保存用户信息
     * @param cmMallAdminUser
     * @return
     */
    @PostMapping(value = "/saveUser")
    public ResponseJson saveUser(@RequestBody CmMallAdminUser cmMallAdminUser) {
        if (StringUtils.isEmpty(cmMallAdminUser.getPassword())) {
            return ResponseJson.error(-1,"密码不能为空",null);
        }
        if (StringUtils.isEmpty(cmMallAdminUser.getOldPassword())) {
            return ResponseJson.error(-1,"请确认密码",null);
        }
        String regex = "^[A-Za-z0-9]{8,16}$";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(cmMallAdminUser.getPassword());
        if (!matcher.matches()) {
            return ResponseJson.error(-1,"密码格式不正确",null);
        }
        if (!cmMallAdminUser.getPassword().equals(cmMallAdminUser.getOldPassword())) {
            return ResponseJson.error(-1,"两次密码不一致",null);
        }
        if (null == cmMallAdminUser.getId()) {
            Integer repeat = mallUserDao.getRepeat(cmMallAdminUser.getAccount(), null);
            if (null != repeat) {
                return ResponseJson.error(-1, "用户名已存在，请勿重复添加", null);
            }
        }
        mallUserService.saveUser(cmMallAdminUser);
        return ResponseJson.success();
    }

    /**
     * 重置密码
     * @param password
     * @param oldPassword
     * @param id
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/repeatPassword")
    public ResponseJson repeatPassword(String password, String oldPassword, Integer id){
        if (StringUtils.isEmpty(password)) {
            return ResponseJson.error(-1,"密码不能为空",null);
        }
        if (StringUtils.isEmpty(oldPassword)) {
            return ResponseJson.error(-1,"确认密码不能为空",null);
        }
        String regex = "^[a-zA-Z0-9]{8,16}$";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(password);
        if (matcher.matches()) {
            return ResponseJson.error(-1,"密码格式不正确",null);
        }
        if (!StringUtils.equals(password,oldPassword)) {
            return ResponseJson.error(-1,"两次密码不相同",null);
        }
        if (null == id) {
            return ResponseJson.error(-1,"用户Id不能为空",null);
        }
        return mallUserService.repeatPassword(password, id);
    }
}
